from fastapi import APIRouter, Cookie, Depends, Body, Query
from ehelply_bootstrapper.drivers.fast_api_utils.responses import *

from typing import List, Dict

import json
from pathlib import Path

from ehelply_bootstrapper.utils.state import State

router = APIRouter()


@router.get(
    '/meta',
    tags=["meta"],
)
async def get_meta():
    """
    Returns the service meta for this microservice
    :return:
    """
    return http_200_ok({
        "version": State.bootstrapper.service_meta.version,
        "name": State.bootstrapper.service_meta.name,
        "summary": State.bootstrapper.service_meta.summary,
        "authors": State.bootstrapper.service_meta.authors,
        "author_emails": State.bootstrapper.service_meta.author_emails
    })


@router.get(
    '/releases',
    tags=["releases"],
)
async def get_releases():
    """
    Returns the release history of this microservice
    :return:
    """
    releases_path: Path = State.bootstrapper.get_service_package_path().joinpath("releases.json")

    with open(str(releases_path)) as file:
        releases_data: list = json.load(file)

    return http_200_ok(releases_data)


@router.get(
    '/releases/{release}',
    tags=["releases"],
)
async def get_release(
        release: str
):
    """
    Returns information about a particular release

    If the release parameter is 'latest', it will be set to the current version of THIS running microservice

    :param release:
    :return:
    """
    if release == "latest":
        release = State.bootstrapper.service_meta.version

    releases_path: Path = State.bootstrapper.get_service_package_path().joinpath("releases.json")

    with open(str(releases_path)) as file:
        releases_data: list = json.load(file)

    for release_from_service in releases_data:
        if release_from_service['version'] == release:
            return http_200_ok(release_from_service)

    return http_404_not_found()


@router.get(
    '/releases/{release}/spec',
    tags=["releases"],
)
async def get_release(
        release: str
):
    """
    Returns the JSON spec of a microservice

    If the release parameter is 'latest', it will be set to the current version of THIS running microservice

    :param release:
    :return:
    """
    if release == "latest":
        release = State.bootstrapper.service_meta.version

    import glob

    docs_path: Path = State.bootstrapper.get_service_package_path().joinpath("docs")

    latest_name: str = ""
    for file_name in glob.glob(str(docs_path) + "/api-spec_{release}_*.json".format(release=release)):

        if Path(file_name).name > latest_name:
            latest_name = Path(file_name).name

    if len(latest_name) == 0:
        return http_404_not_found()

    try:
        with open(str(docs_path.joinpath(latest_name))) as file:
            spec = json.load(file)

        return spec

    except:
        return http_404_not_found()


@router.get(
    '/template',
    tags=["releases"],
)
async def get_service_template_details():
    """
    Returns details about the current service template this microservice is using such as the version
    :return:
    """
    package_path: Path = State.bootstrapper.get_service_package_path().joinpath("ehelply-package.json")

    with open(str(package_path)) as file:
        package_data: dict = json.load(file)

    return package_data



import typer

cli = typer.Typer()


@cli.command()
def migrate():
    """
    Run all SQLAlchemy migrations in the current environment
    """

    typer.echo("Running Migrations..")
    from alembic.config import Config
    import alembic.command

    config = Config('alembic.ini')
    config.attributes['configure_logger'] = False

    alembic.command.upgrade(config, 'head')


@cli.command()
def create_migration(
        message: str = typer.Option(..., prompt=True, help="A brief message describing what is included in this migration")
):
    """
    Create a new SQLAlchemy migration in the current environment
    """

    typer.echo("Creating migration..")
    from alembic.config import Config
    import alembic.command

    config = Config('alembic.ini')
    config.attributes['configure_logger'] = False

    alembic.command.revision(config, autogenerate=True, message=message)


if __name__ == '__main__':
    cli()



import math

from sqlalchemy.orm.query import Query
from pydantic import BaseModel
from typing import Callable, Union


class Pagination(BaseModel):
    """
    Pagination state
    """
    current_page: int
    page_size: int
    total_items: int
    total_pages: int
    has_previous_page: bool
    has_next_page: bool
    previous_page: Union[int, None]
    next_page: Union[int, None]


class Page(BaseModel):
    """
    Page state
    """
    items: list
    pagination: Pagination

    def transform(self, transformer: Callable):
        """
        Transforms results from a DB backbone into a Pydantic (or other) object type

        Args:
            transformer (Callable): A function which takes in an input and returns a transformed output

        Returns:
            None

        """
        items: list = self.items
        self.items = []
        for item in items:
            self.items.append(transformer(item))


class PageMaker:
    """
    Automates the process of creating page state
    """
    def __init__(self, items, page: int, page_size: int, total: int):
        self.items: list = items
        self.total: int = total
        self.previous_page: int = None
        self.next_page: int = None
        self.has_previous: bool = page > 1
        self.current_page: int = page

        self.page_size = page_size
        if self.page_size > 100:
            self.page_size = 100

        if self.has_previous:
            self.previous_page = page - 1

        previous_items: int = (page - 1) * page_size
        self.has_next: bool = previous_items + len(items) < total

        if self.has_next:
            self.next_page = page + 1

        self.pages: int = int(math.ceil(total / float(page_size)))

    def make(self) -> Page:
        """
        Makes a new page

        Returns:
            Page: The current page state

        """
        return Page(
            items=self.items,
            pagination=Pagination(
                current_page=self.current_page,
                page_size=self.page_size,
                total_items=self.total,
                total_pages=self.pages,
                has_previous_page=self.has_previous,
                has_next_page=self.has_next,
                previous_page=self.previous_page,
                next_page=self.next_page
            )
        )


class SQLPaginator:
    """
    Use this class to utilize pagination for SQL DBs
    """
    def __init__(self, page: int, page_size: int):
        """

        Args:
            page (int): The page number to return results for
            page_size (int): The number of results to include in a page
        """
        self.page = page
        self.page_size = page_size
        if self.page_size > 100:
            self.page_size = 100

    def paginate(self, query: Query) -> Page:
        """
        Takes in a SQL Alchemy query and paginates the result.

        Args:
            query (Query): This should be a completed query without a limit or offset applied.

        Returns:
            Page: The page state

        """
        if self.page <= 0:
            raise AttributeError('page needs to be >= 1')

        if self.page_size <= 0:
            raise AttributeError('page_size needs to be >= 1')

        items: list = query.limit(self.page_size).offset((self.page - 1) * self.page_size).all()

        # We remove the ordering of the query since it doesn't matter for getting a count and
        # might have performance implications as discussed on this Flask-SqlAlchemy issue
        # https://github.com/mitsuhiko/flask-sqlalchemy/issues/100
        total: int = query.order_by(None).count()

        return PageMaker(items, self.page, self.page_size, total).make()
